/* par.h - define parameter pattern indices

	Copyright (c) 1984 by JMI Software Consultants, Inc.

These are the parameter pattern definitions.  Each line has four
parts to it (after the keyword define).  The first is the name
of the parameter pattern; the second is the index in the array pdt
in the file parampat.c that starts that particular pattern; 
the third is the description to show the way the input will look;
and the fourth is the description to show the way the output will look.

Abbreviations used:
	I -- Integer expression
	D -- Double expression
	S -- String expression
	A -- Any expression
	P -- Pointer to a string
	F -- FIELD variable
	C -- Integer constant 0 - 9
	L -- Current statement number
	a|b -- either a or b
	' ' -- Literal
	[ ]	-- Optional.  If what is looked for isn't found, the
			default value for that type is outputted (except with ANY 
			or single letters, nothing is outputted).
	@TS -- Address of Temporary String
	... -- An infinite list of the same type, with the last item
			of the list being the default value of that type.
*/

#define par_I		1	/* (I)					(I)						*/
#define par_D		4	/* (D)					(D)						*/
#define par_S		6	/* (S)					(S)						*/
#define par_INSTR	8	/* ([I,] S, S)			(I, S, S)				*/
#define par_OPT		14	/* (A)					(A)						*/
#define par_1OINT	16	/* [I]					(I)						*/
#define par_3OINT	18	/* [I, I, I]			(I, I, I)				*/
#define par_5OINT	20	/* [I, I, I, I, I]		(I, I, I, I, I)			*/
#define par_USR		32	/* [C]=D				'D'(I, D)				*/
#define par_VPTR	33	/* [#I]|A				(I, A)					*/
#define par_DATE	38	/* =S					'S'(S)					*/
#define par_FN		39	/* (A, ...)				(A, ...)				*/
#define par_TNARG	43	/* 						(@TS)					*/
#define par_LSET	45	/* F|S=S				(F|P, S)				*/
#define par_NARG	48	/* 						()						*/
#define par_BINF	49	/* [#][D,] ...			(I, ..., 0)				*/
#define par_MDST	56	/* (S, I [,I])=S		'S'(P, I, I, S)			*/
#define par_NAME	67	/* S AS S				(S, S)					*/
#define par_X1OINT	71	/* I					(I)						*/
#define par_WIDTH	72	/* S|I, I				['D'](S|I, I)			*/
#define par_INP		81	/* (I [,I])				(@TS, I, I)			*/
#define par_FUSR	86	/* [C](A)				(I, A)					*/
#define par_SOPT	93	/* [S]					(S)						*/
#define par_WAIT	96	/* I, I [,I]			(I, I, I)				*/
#define par_TI		100	/* (I)					(@TS, I)				*/
#define par_TSI		103	/* (S, I)				(@TS, S, I)			*/
#define par_TD		106	/* (D)					(@TS, D)				*/
#define par_STR		111	/* (I, I|S)				['D'](@TS, I|S)		*/
#define par_MID		118	/* (S, I [,I])			(@TS, S, I, I)			*/
#define par_XS		121	/* S					(S)						*/
#define par_XI		124	/* I					(I)						*/
#define par_XUI		125	/* I, I					(I, I)					*/
#define par_SEG		129	/* [=][D]				(D)						*/
#define par_OPEN	131	/* S, I, S [,I]			'B'(S, I, S, I)			*/
						/* or											*/
						/* S [FOR mode] AS I [LEN=I]	'BS'(S, I, I, I)*/
						/* mode can be APPEND, INPUT or OUTPUT			*/
#define par_GET		159	/* [#]I [,I]			(I, I)					*/
#define par_LNUM	162	/*						(L)						*/
#define par_RND		164	/* [(D)]				(D)						*/
#define par_RAN		166	/* [D]					'S'(D)					*/
#define par_CLEAR	178	/* [, D, D]				(I, D, D)				*/
#define par_BU		184	/* (I)					'B'(I)					*/
#define par_BXUI	185	/* I, I					'B'(I, I)				*/
#define par_SHELL	191
#define par_LOCK	194
#define par_UNLK	205
